@@ -8,19 +8,18 @@ import logging
 import operator
 
 from .py27compat import configparser, filter
-from .py33compat import max
 
-from . import logger
 from . import backend
 from .util import platform_ as platform
-from .util import once
 from .backends import fail
+from .backends.chainer import ChainerBackend
 
 
 log = logging.getLogger(__name__)
 
 _keyring_backend = None
 
+
 def set_keyring(keyring):
     """Set current keyring backend.
     """
@@ -36,6 +35,23 @@ def get_keyring():
     return _keyring_backend
 
 
+def disable():
+    """
+    Configure the null keyring as the default.
+    """
+    root = platform.config_root()
+    try:
+        os.makedirs(root)
+    except OSError:
+        pass
+    filename = os.path.join(root, 'keyringrc.cfg')
+    if os.path.exists(filename):
+        msg = "Refusing to overwrite {filename}".format(**locals())
+        raise RuntimeError(msg)
+    with open(filename, 'w') as file:
+        file.write('[backend]\ndefault-keyring=keyring.backends.null.Keyring')
+
+
 def get_password(service_name, username):
     """Get password from the specified service.
     """
@@ -54,20 +70,43 @@ def delete_password(service_name, username):
     _keyring_backend.delete_password(service_name, username)
 
 
-recommended = lambda backend: backend.priority >= 1
+def get_credential(service_name, username):
+    """Get a Credential for the specified service.
+    """
+    return _keyring_backend.get_credential(service_name, username)
+
+
+def recommended(backend):
+    return backend.priority >= 1
+
+
 by_priority = operator.attrgetter('priority')
 
 
+def _create_chained_backend(keyrings):
+    """
+    If one keyring is available, returns it. If more than one, returns a
+    single keyring that chains them. If no keyrings, returns None.
+    """
+    keyrings = sorted((k for k in keyrings if k.priority >= 1),
+                      key=by_priority, reverse=True)
+    if keyrings:
+        if len(keyrings) == 1:
+            return keyrings[0]
+        return ChainerBackend(keyrings)
+
+
 def init_backend(limit=None):
     """
     Load a keyring specified in the config file or infer the best available.
     """
-    _load_library_extensions()
     keyrings = filter(limit, backend.get_all_keyring())
 
     set_keyring(
-        load_config()
-        or max(keyrings, default=fail.Keyring, key=by_priority)
+        load_env()
+        or load_config()
+        or _create_chained_backend(keyrings)
+        or fail.Keyring()
     )
 
 
@@ -110,6 +149,14 @@ def load_keyring(keyring_name):
     return class_()
 
 
+def load_env():
+    """Load a keyring configured in the environment variable."""
+    try:
+        return load_keyring(os.environ['PYTHON_KEYRING_BACKEND'])
+    except KeyError:
+        pass
+
+
 def load_config():
     """Load a keyring using the config file in the config root."""
 
@@ -132,12 +179,14 @@ def load_config():
             raise configparser.NoOptionError('backend', 'default-keyring')
 
     except (configparser.NoOptionError, ImportError):
-        logger.warning("Keyring config file contains incorrect values.\n" +
-                       "Config file: %s" % keyring_cfg)
+        logger = logging.getLogger('keyring')
+        logger.warning("Keyring config file contains incorrect values.\n"
+                       + "Config file: %s" % keyring_cfg)
         return
 
     return load_keyring(keyring_name)
 
+
 def _load_keyring_path(config):
     "load the keyring-path option (if present)"
     try:
@@ -146,40 +195,6 @@ def _load_keyring_path(config):
     except (configparser.NoOptionError, configparser.NoSectionError):
         pass
 
-@once
-def _load_library_extensions():
-    """
-    Locate all setuptools entry points by the name 'keyring backends'
-    and initialize them.
-
-    Any third-party library may register an entry point by adding the
-    following to their setup.py::
-
-        entry_points = {
-            'keyring backends': [
-                'plugin name = mylib.mymodule:initialize_func',
-            ],
-        },
-
-    `plugin name` can be anything.
-    `initialize_func` is optional and will be invoked by keyring on startup.
-
-    Most plugins will simply provide or import a KeyringBackend in `mymodule`.
-    """
-    group = 'keyring backends'
-    try:
-        pkg_resources = __import__('pkg_resources')
-    except ImportError:
-        return
-    entry_points = pkg_resources.iter_entry_points(group=group)
-    for ep in entry_points:
-        try:
-            log.info('Loading keyring backends from %s', ep.name)
-            init_func = ep.load()
-            if callable(init_func):
-                init_func()
-        except Exception as exc:
-            log.exception("Error initializing plugin %s (%s).", ep, exc)
 
 # init the _keyring_backend
 init_backend()
